Package: branchpointer
Type: Package
Title: Prediction of intronic splicing branchpoints
Version: 1.38.0
Date: 2025-07-22
Author: Beth Signal
Maintainer: Beth Signal <b.signal@garvan.org.au>
Description: Predicts branchpoint probability for sites in intronic branchpoint windows. Queries can be supplied as intronic regions; or to evaluate the effects of mutations, SNPs.
License: BSD_3_clause + file LICENSE
Depends: caret, R(>= 3.4)
Imports: plyr, kernlab, gbm, stringr, cowplot, ggplot2, biomaRt,
        Biostrings, parallel, utils, stats,
        BSgenome.Hsapiens.UCSC.hg38, rtracklayer, GenomicRanges,
        Seqinfo, IRanges, S4Vectors, data.table
Suggests: knitr, BiocStyle
RoxygenNote: 6.0.1
VignetteBuilder: knitr
biocViews: Software, GenomeAnnotation, GenomicVariation,
        MotifAnnotation
git_url: https://git.bioconductor.org/packages/branchpointer
git_branch: RELEASE_3_23
git_last_commit: 8b735ec
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:59:07 UTC; biocbuild
Built: R 4.6.1; ; 2026-08-13 16:40:41 UTC; unix
