# Generated by roxygen2: do not edit by hand

export(gtfToExons)
export(makeBranchpointWindowForExons)
export(makeBranchpointWindowForSNP)
export(plotBranchpointWindow)
export(predictBranchpoints)
export(predictionsToSummary)
export(readQueryFile)
import(GenomicRanges)
import(biomaRt)
import(caret)
import(gbm)
import(ggplot2)
import(parallel)
import(plyr)
import(stringr)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(data.table,data.table)
importClassesFrom(kernlab,ksvm)
importFrom(BSgenome.Hsapiens.UCSC.hg38,BSgenome.Hsapiens.UCSC.hg38)
importFrom(Biostrings,complement)
importFrom(Biostrings,getSeq)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,Rle)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(cowplot,draw_plot)
importFrom(cowplot,ggdraw)
importFrom(data.table,fread)
importFrom(kernlab,predict)
importFrom(rtracklayer,import)
importFrom(stats,aggregate)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(utils,write.table)
